form.releditForm div.iformTitle {
  background: transparent;
  border: none;
  margin-top: 1em;
  margin-bottom: 0.5em;
}

div.iformTitle span {
  font-weight: bold;
}

form.releditForm div.iformBody label {
  font-weight: normal;
  padding-left: 1em;
}

div.btn-group {
  margin-bottom: 1em;
}

.metaField .control-label {
  font-weight: normal;
  width: 20%;
}

.jqtreeNodeSelected {
  font-weight: bold;
}

.SEDAArchiveUnit > a {
  color: #06A3A3;
}

.SEDABinaryDataObject > a, .SEDAPhysicalDataObject > a {
  color: #AE8332;
}

.cw-table-primary-entity td {
  width: 67%;
}